# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
# Copyright (C) 2021 Northwestern University.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Sort parameter interpreter API."""

from functools import partial

from invenio_records_resources.services.records.params.base import \
    ParamInterpreter


class AllVersionsParam(ParamInterpreter):
    """Evaluates the 'allversions' parameter."""

    def __init__(self, field_name, config):
        """Construct."""
        self.field_name = field_name
        super().__init__(config)

    @classmethod
    def factory(cls, field):
        """Create a new filter parameter."""
        return partial(cls, field)

    def apply(self, identity, search, params):
        """Evaluate the allversions parameter on the search."""
        if not params.get("allversions"):
            search = search.filter('term', **{self.field_name: True})
        return search



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.
#

"""Base class for service components."""

from invenio_records_resources.services.records.components import \
    ServiceComponent as BaseServiceComponent


class ServiceComponent(BaseServiceComponent):
    """Base service component."""

    def read_draft(self, identity, draft=None):
        """Update draft handler."""
        pass

    def update_draft(self, identity, data=None, record=None, errors=None):
        """Update draft handler."""
        pass

    def delete_draft(self, identity, draft=None, record=None, force=False):
        """Delete draft handler."""
        pass

    def edit(self, identity, draft=None, record=None):
        """Edit a record handler."""
        pass

    def new_version(self, identity, draft=None, record=None):
        """New version handler."""
        pass

    def publish(self, identity, draft=None, record=None):
        """Publish handler."""
        pass

    def import_files(self, identity, draft=None, record=None):
        """Import files handler."""
        pass



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Error classes and handlers."""

import json

from flask_resources.errors import HTTPJSONException


class RedirectException(HTTPJSONException):
    """Trigger a redirect to the latest record version.

    The `create_latest_version_redirect_error_handler` handles this exception
    defined in the `RecordVersionsResourceConfig.error_map`. The handler is
    redirecting to the location instructed by the raised Exception.
    """

    code = 301

    def __init__(self, location, **kwargs):
        """Constructor."""
        self.location = location
        kwargs.setdefault("description", "Redirecting...")
        super().__init__(**kwargs)

    def get_headers(self, environ=None):
        """Get response headers."""
        return [
            ("Content-Type", "application/json"), ("Location", self.location)]

    def get_body(self, environ=None):
        """Get the request body."""
        body = {
            "status": self.code,
            "message": self.get_description(environ),
            "location": self.location
        }
        return json.dumps(body)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
# Copyright (C) 2021 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.


from .utils import create_and_publish


def create_two_versions(service, identity_simple, input_data):
    """Create 2 versions."""
    record = create_and_publish(service, identity_simple, input_data)
    draft = service.new_version(record.id, identity_simple)
    # new_version resets files.enabled to True
    service.update_draft(draft.id, identity_simple, input_data)
    record_2 = service.publish(draft.id, identity_simple)
    service.record_cls.index.refresh()
    return [record, record_2]


def test_search_versions(app, identity_simple, input_data, service):
    v1, v2 = create_two_versions(service, identity_simple, input_data)

    # test search defaults to latest version
    results = service.search(identity_simple)
    assert set([v2.id]) == set([r["id"] for r in results])

    # test search all versions
    results = service.search(identity_simple, params={"allversions": True})
    assert set([v1.id, v2.id]) == set([r["id"] for r in results])
